! ===========================================================================
MODULE act_likelihood_lens
! ===========================================================================


  integer :: lmin                 =  50   !minimum ell to consider
  integer :: lmax                 =  2050 !maximum ell to consider
  integer, parameter :: bmaxl       = 5   !max number of data points  

  character(len=*), parameter :: ACT_lens_dir = 'data/'
  character(len=*), parameter, public :: act_likelihood_version='v2_lens'
  
  logical :: initialise_act=.true.
  REAL(8) :: bpp_dat(0:bmaxl-1),bpp_var(0:bmaxl-1),bval(0:bmaxl-1)
  integer(8), dimension (:), allocatable :: blmin, blmax
  integer :: bmin, bmax

  PRIVATE
  public :: act_likelens_init
  public :: act_likelens_compute
  public :: get_free_lun
contains
  
  ! ===========================================================================
  SUBROUTINE act_likelens_init 
    ! ===========================================================================
    
    IMPLICIT NONE
    integer  :: i,lun
    character(LEN=240) :: clfilename, binfilename
    real :: jmin,jmax

    allocate(blmax(0:bmaxl-1), blmin(0:bmaxl-1))
    
    print *, ''
    print *, 'Initializing ACT likelihood, version '//act_likelihood_version
    
    clfilename  = trim(ACT_lens_dir)//'cl_kk_act_v2.dat'
    binfilename= trim(ACT_lens_dir)//'bin_kk_act_v2.dat'
 
   call get_free_lun( lun )
    open(unit=lun,file=clfilename,form='formatted',status='unknown',action='read')
    
    do i=0,bmaxl-1
       read(lun,*) bval(i),bpp_dat(i), bpp_var(i)
       bpp_var(i)=bpp_var(i)**2
    enddo
    close(lun)
    !Get bin range to be used, using lrange set at top
    do i=0,bmaxl-1
       if(bval(i) <= lmin) bmin=i+1
       if(bval(i) <= lmax) bmax=i
    enddo

    call get_free_lun( lun )

    open(unit=lun,file=binfilename,form='formatted',status='unknown',action='read')
    do i = 0,bmaxl-1
       read(lun,*) blmin(i), blmax(i)
    end do
    close(lun)
   initialise_act = .false.
    

  END SUBROUTINE act_likelens_init
  
  ! ===========================================================================
  
  SUBROUTINE act_likelens_compute(clpp,like)
    
    ! ===========================================================================
    
    IMPLICIT NONE
    REAL(8), intent(in) :: clpp(2:*)
    REAL(8),intent(out) :: like
    INTEGER :: il   
    REAL(8) :: bpp_th(0:bmaxl-1),diffs(0:bmaxl-1),dlnlike,ell(2:blmax(bmaxl-1))

    Like = 0.d0
    do il=2,blmax(bmaxl-1)
       ell(il)=il*1.d0
    enddo

    do il=0,bmaxl-1  !Do ell weighted average in each bin
      bpp_th(il)=SUM(ell(blmin(il):blmax(il))*clpp(int(blmin(il)):int(blmax(il))))/SUM(ell(blmin(il):blmax(il)))
     enddo
	
    diffs(bmin:bmax) = bpp_dat(bmin:bmax) - bpp_th(bmin:bmax)
    dlnlike = SUM(diffs(bmin:bmax)**2/bpp_var(bmin:bmax)) 
    Like = dlnlike/2.d0
    
10  continue
    
  end SUBROUTINE act_likelens_compute
  


  subroutine get_free_lun( lun )

        implicit none
        integer, intent(out) :: lun

        integer, save :: last_lun = 19
        logical :: used
        lun = last_lun
        do
                inquire( unit=lun, opened=used )
                if ( .not. used ) exit
                lun = lun + 1
        end do

        last_lun = lun
          end subroutine


END MODULE act_likelihood_lens

